@@ -1,9 +1,12 @@
 import collections
 import statistics
+from typing import Iterable
+
 import sklearn.metrics as sm
 import numpy as np
 from abc import ABC, abstractmethod
 
+import cv2
 from pycocotools.cocoeval import Params, COCOeval
 from sklearn.metrics import balanced_accuracy_score
 
@@ -11,6 +14,13 @@ from .prediction_filters import TopKPredictionFilter, ThresholdPredictionFilter
 from functools import reduce
 
 
+def _indices_to_vec(indices, length):
+    target_vec = np.zeros(length, dtype=int)
+    target_vec[indices] = 1
+
+    return target_vec
+
+
 def _targets_to_mat(targets, n_class):
     if len(targets.shape) == 1:
         target_mat = np.zeros((len(targets), n_class), dtype=int)
@@ -27,6 +37,7 @@ class Evaluator(ABC):
     """
 
     def __init__(self):
+        self.custom_fields = {}
         self.reset()
 
     @abstractmethod
@@ -67,6 +78,9 @@ class MemorizingEverythingEvaluator(Evaluator, ABC):
     """
 
     def __init__(self, prediction_filter=None):
+        self.all_targets = np.array([])
+        self.all_predictions = np.array([])
+
         super(MemorizingEverythingEvaluator, self).__init__()
         self.prediction_filter = prediction_filter
 
@@ -84,18 +98,17 @@ class MemorizingEverythingEvaluator(Evaluator, ABC):
 
         assert len(predictions) == len(targets)
 
-        target_mat = _targets_to_mat(targets, predictions.shape[1])
         predictions = self.prediction_filter.filter(predictions, 'vec') if self.prediction_filter else predictions
 
         if self.all_predictions.size != 0:
             self.all_predictions = np.append(self.all_predictions, predictions, axis=0)
         else:
-            self.all_predictions = np.copy(predictions)
+            self.all_predictions = predictions.copy()
 
         if self.all_targets.size != 0:
-            self.all_targets = np.append(self.all_targets, target_mat, axis=0)
+            self.all_targets = np.append(self.all_targets, targets, axis=0)
         else:
-            self.all_targets = np.copy(target_mat)
+            self.all_targets = targets.copy()
 
     def calculate_score(self, average='macro'):
         """
@@ -115,12 +128,16 @@ class MemorizingEverythingEvaluator(Evaluator, ABC):
         ``'samples'``:
             Calculate metrics for each instance, and find their average.
         """
-        assert self.all_targets.size == self.all_predictions.size
+        if self.all_predictions.size == 0:
+            return 0.0
+
+        tar_mat = _targets_to_mat(self.all_targets, self.all_predictions.shape[1])
+        assert tar_mat.size == self.all_predictions.size
         result = 0.0
-        if self.all_targets.size > 0:
-            non_empty_idx = np.where(np.invert(np.all(self.all_targets == 0, axis=0)))[0]
+        if tar_mat.size > 0:
+            non_empty_idx = np.where(np.invert(np.all(tar_mat == 0, axis=0)))[0]
             if non_empty_idx.size != 0:
-                result = self._calculate(self.all_targets[:, non_empty_idx], self.all_predictions[:, non_empty_idx], average=average)
+                result = self._calculate(tar_mat[:, non_empty_idx], self.all_predictions[:, non_empty_idx], average=average)
 
         return result
 
@@ -133,7 +150,7 @@ class MemorizingEverythingEvaluator(Evaluator, ABC):
         pass
 
     def get_report(self, **kwargs):
-        average = kwargs['average'] if 'average' in kwargs else 'macro'
+        average = kwargs.get('average', 'macro')
         return {self._get_id(): self.calculate_score(average)}
 
 
@@ -142,9 +159,13 @@ class TopKAccuracyEvaluator(Evaluator):
     Top k accuracy evaluator for multiclass classification
     """
 
-    def __init__(self, k):
-        self.prediction_filter = TopKPredictionFilter(k)
+    def __init__(self, k: int):
+        assert k > 0
+        self.total_num = 0
+        self.topk_correct_num = 0
+
         super(TopKAccuracyEvaluator, self).__init__()
+        self.prediction_filter = TopKPredictionFilter(k)
 
     def reset(self):
         super(TopKAccuracyEvaluator, self).reset()
@@ -179,6 +200,9 @@ class ThresholdAccuracyEvaluator(Evaluator):
     """
 
     def __init__(self, threshold):
+        self.num_sample = 0
+        self.sample_accuracy_sum = 0
+
         super().__init__()
         self.prediction_filter = ThresholdPredictionFilter(threshold)
 
@@ -220,7 +244,7 @@ class F1ScoreEvaluator(EvaluatorAggregator):
         self._filter_id = prediction_filter.identifier
 
     def get_report(self, **kwargs):
-        average = kwargs['average'] if 'average' in kwargs else 'macro'
+        average = kwargs.get('average', 'macro')
         report = super(F1ScoreEvaluator, self).get_report(average=average)
         prec = report[f'precision_{self._filter_id}']
         recall = report[f'recall_{self._filter_id}']
@@ -270,6 +294,33 @@ class AveragePrecisionEvaluator(MemorizingEverythingEvaluator):
     def _get_id(self):
         return 'average_precision'
 
+    def calculate_score(self, average='macro'):
+        if average != 'macro':
+            return super().calculate_score(average)
+
+        ap = 0.0
+        if self.all_targets.size == 0:
+            return ap
+
+        n_class_with_gt = 0
+        c_to_sample_indices = dict()
+        for sample_idx, targets in enumerate(self.all_targets):
+            if isinstance(targets, Iterable):
+                for t_idx, t in enumerate(targets):
+                    if t:
+                        c_to_sample_indices.setdefault(t_idx, []).append(sample_idx)
+            else:
+                c_to_sample_indices.setdefault(targets, []).append(sample_idx)
+
+        for c_idx in range(self.all_predictions.shape[1]):
+            if c_idx not in c_to_sample_indices:
+                continue
+            class_target_vec = _indices_to_vec(c_to_sample_indices[c_idx], len(self.all_targets))
+            ap += sm.average_precision_score(class_target_vec, self.all_predictions[:, c_idx])
+            n_class_with_gt += 1
+
+        return ap / n_class_with_gt if n_class_with_gt > 0 else 0.0
+
     def _calculate(self, targets, predictions, average):
         return sm.average_precision_score(targets, predictions, average=average)
 
@@ -374,6 +425,44 @@ class EceLossEvaluator(Evaluator):
         self.sum_confidence_in_bin = np.zeros(self.n_bins)
 
 
+class RocAucEvaluator(Evaluator):
+    """
+    Utilize sklearn.metrics.roc_auc_score to Compute Area Under the Receiver Operating Characteristic Curve (ROC AUC) from prediction scores.
+
+    Check https://scikit-learn.org/stable/modules/generated/sklearn.metrics.roc_auc_score.html#sklearn.metrics.roc_auc_score for details.
+    """
+
+    def reset(self):
+        super(RocAucEvaluator, self).reset()
+        self.all_targets = None
+        self.all_predictions = None
+
+    def add_predictions(self, predictions, targets):
+        """ add predictions and targets.
+        Args:
+            predictions: predictions of array-like of shape (n_samples,) or (n_samples, n_classes)
+            targets: targets of array-like of shape (n_samples,) or (n_samples, n_classes)
+
+        """
+        self.all_targets = np.concatenate([self.all_targets, np.array(targets)]) if self.all_targets else np.array(targets)
+        self.all_predictions = np.concatenate([self.all_predictions, np.array(predictions)]) if self.all_predictions else np.array(predictions)
+
+    def get_report(self, **kwargs):
+        average = kwargs.get('average', 'macro')
+        sample_weight = kwargs.get('sample_weight')
+        max_fpr = kwargs.get('max_fpr')
+        multi_class = kwargs.get('multi_class', 'raise')
+        labels = kwargs.get('labels')
+
+        if len(self.all_targets.shape) == 1 and len(self.all_predictions.shape) == 2 and self.all_predictions.shape[1] == 2:
+            all_predictions = self.all_predictions[:, 1]
+        else:
+            all_predictions = self.all_predictions
+        return {
+            'roc_auc': sm.roc_auc_score(y_true=self.all_targets, y_score=all_predictions, average=average, sample_weight=sample_weight, max_fpr=max_fpr, multi_class=multi_class, labels=labels)
+        }
+
+
 class MeanAveragePrecisionEvaluatorForSingleIOU(Evaluator):
     def __init__(self, iou=0.5, report_tag_wise=False):
         """
@@ -481,7 +570,7 @@ class MeanAveragePrecisionEvaluatorForSingleIOU(Evaluator):
         return sm.average_precision_score(is_correct, probabilities, average=average) * recall
 
     def get_report(self, **kwargs):
-        average = kwargs['average'] if 'average' in kwargs else 'macro'
+        average = kwargs.get('average', 'macro')
         for class_index in self.is_correct:
             ap = MeanAveragePrecisionEvaluatorForSingleIOU._calculate_average_precision(self.is_correct[class_index], self.probabilities[class_index], self.true_num[class_index], average)
             self.aps[class_index] = ap
@@ -604,7 +693,31 @@ class CocoMeanAveragePrecisionEvaluator(Evaluator):
             mean_s = np.mean(s[s > -1])
         return mean_s
 
+    def remap_cat_ids(self):
+        """Map target category ids to continuous ones to avoid the index overflow issue during coco mAP computation.
+           Then any predicted category ids not shown in targets will be appended. e.g. targets:[0,2,4], predictions:[0,1,2,3],
+           the category ids are remapped as 0->0, 2->1, 4->2, 1(not shown in targets)->3, 3(not shown in targets)->4"""
+
+        # Create the map.
+        target_cat_ids = set([b[0] for bboxes in self.targets for b in bboxes])
+        predict_cat_ids = set([b[0] for bboxes in self.predictions for b in bboxes])
+        cate_id_new_to_old = sorted(list(target_cat_ids)) + sorted(list(predict_cat_ids - target_cat_ids))
+        cate_id_old_to_new = {o: n for n, o in enumerate(cate_id_new_to_old)}
+
+        # Apply the map.
+        def _apply_cate_id_map(gt_or_pred):
+            for bboxes in gt_or_pred:
+                for bbox in bboxes:
+                    bbox[0] = cate_id_old_to_new[bbox[0]]
+
+        _apply_cate_id_map(self.targets)
+        _apply_cate_id_map(self.predictions)
+
+        # Return the new id to old id map.
+        return cate_id_new_to_old
+
     def get_report(self, **kwargs):
+        cat_id_new_to_old_map = self.remap_cat_ids()
         coco_eval_result = self._coco_eval()
         report = {'avg_mAP': self._summarize(coco_eval_result, 1, maxDets=coco_eval_result.params.maxDets[-1])}
         # mAP for each iou
@@ -613,8 +726,8 @@ class CocoMeanAveragePrecisionEvaluator(Evaluator):
         # tag-wise mAP
         for iou, iou_report_tag_wise in zip(coco_eval_result.params.iouThrs, self.report_tag_wise):
             if iou_report_tag_wise:
-                report[f'tag_wise_AP_{int(iou * 100)}'] = [self._summarize(coco_eval_result, 1, iou, maxDets=coco_eval_result.params.maxDets[-1], catId=cat_id) for cat_id in
-                                                           coco_eval_result.params.catIds]
+                report[f'tag_wise_AP_{int(iou * 100)}'] = {cat_id_new_to_old_map[cat_id]: self._summarize(coco_eval_result, 1, iou, maxDets=coco_eval_result.params.maxDets[-1], catId=cat_id)
+                                                           for cat_id in coco_eval_result.params.catIds}
 
         return report
 
@@ -626,7 +739,7 @@ class CocoMeanAveragePrecisionEvaluator(Evaluator):
 
 class BalancedAccuracyScoreEvaluator(MemorizingEverythingEvaluator):
     """
-    Average of recall obtained on each class, for multiclass classifiation problem
+    Average of recall obtained on each class, for multiclass classification problem
     """
 
     def _calculate(self, targets, predictions, average):
@@ -675,3 +788,316 @@ class MeanAveragePrecisionNPointsEvaluator(MemorizingEverythingEvaluator):
 
     def _get_id(self):
         return f'mAP_{self.n_points}_points'
+
+
+class ImageCaptionEvaluatorBase(Evaluator):
+    """
+    Base class for image caption metric evaluator
+    """
+
+    def __init__(self, metric):
+        self.targets = []
+        self.predictions = []
+        super(ImageCaptionEvaluatorBase, self).__init__()
+        self.metric = metric
+
+    def add_predictions(self, predictions, targets):
+        """ Evaluate list of image with image caption results using pycocoimcap tools.
+        Args:
+            predictions: list of string predictions [caption1, caption2, ...], shape: (N, ), type: string
+            targets: list of string ground truth for image caption task: [[gt1, gt2, ...], [gt1, gt2, ...], ...], type: string
+        """
+        self.targets += targets
+        self.predictions += predictions
+
+    def reset(self):
+        super(ImageCaptionEvaluatorBase, self).reset()
+        self.targets = []
+        self.predictions = []
+
+    def get_report(self, **kwargs):
+        from .coco_evalcap_utils import ImageCaptionCOCOEval, ImageCaptionCOCO, ImageCaptionWrapper
+        imcap_predictions, imcap_targets = ImageCaptionWrapper.convert(self.predictions, self.targets)
+        coco = ImageCaptionCOCO(imcap_targets)
+        cocoRes = coco.loadRes(imcap_predictions)
+        cocoEval = ImageCaptionCOCOEval(coco, cocoRes, self.metric)
+        cocoEval.params['image_id'] = cocoRes.getImgIds()
+        cocoEval.evaluate()
+        result = cocoEval.eval
+        return result
+
+
+class BleuScoreEvaluator(ImageCaptionEvaluatorBase):
+    """
+    BLEU score evaluator for image caption task. For more details, refer to http://www.aclweb.org/anthology/P02-1040.pdf.
+    """
+
+    def __init__(self):
+        super().__init__(metric='Bleu')
+        self.predictions = []
+        self.targets = []
+
+
+class METEORScoreEvaluator(ImageCaptionEvaluatorBase):
+    """
+    METEOR score evaluator for image caption task. For more details, refer to http://www.cs.cmu.edu/~alavie/METEOR/.
+    """
+
+    def __init__(self):
+        super().__init__(metric='METEOR')
+        self.predictions = []
+        self.targets = []
+
+
+class ROUGELScoreEvaluator(ImageCaptionEvaluatorBase):
+    """
+    ROUGE_L score evaluator for image caption task. For more details, refer to http://anthology.aclweb.org/W/W04/W04-1013.pdf
+    """
+
+    def __init__(self):
+        super().__init__(metric='ROUGE_L')
+        self.predictions = []
+        self.targets = []
+
+
+class CIDErScoreEvaluator(ImageCaptionEvaluatorBase):
+    """
+    CIDEr score evaluator for image caption task. For more details, refer to http://arxiv.org/pdf/1411.5726.pdf.
+    """
+
+    def __init__(self):
+        super().__init__(metric='CIDEr')
+        self.predictions = []
+        self.targets = []
+
+
+class SPICEScoreEvaluator(ImageCaptionEvaluatorBase):
+    """
+    SPICE score evaluator for image caption task. For more details, refer to https://arxiv.org/abs/1607.08822.
+    """
+
+    def __init__(self):
+        super().__init__(metric='SPICE')
+        self.predictions = []
+        self.targets = []
+
+
+class MattingEvaluatorBase(Evaluator):
+    """
+    Base class for image matting evaluator
+    """
+
+    def __init__(self, metric):
+        super(MattingEvaluatorBase, self).__init__()
+        self._metric = metric
+        self._num_samples = 0
+        self._metric_sum = 0
+
+    def reset(self):
+        super(MattingEvaluatorBase, self).reset()
+        self._num_samples = 0
+        self._metric_sum = 0
+
+    def _convert2binary(self, mask, threshold=128):
+        bin_mask = mask >= threshold
+        return bin_mask.astype(mask.dtype)
+
+    def _preprocess(self, pred_mask, gt_mask):
+        pred_mask = np.asarray(pred_mask)
+        gt_mask = np.asarray(gt_mask)
+        pred_binmask = self._convert2binary(pred_mask)
+        gt_binmask = self._convert2binary(gt_mask)
+        return pred_binmask, gt_binmask
+
+    def _find_contours(self, matting, thickness=10):
+        matting = np.copy(matting)
+        opencv_major_version = int(cv2.__version__.split('.')[0])
+        if opencv_major_version >= 4:
+            contours, _ = cv2.findContours(matting, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
+        else:
+            _, contours, _ = cv2.findContours(matting, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
+        mask = np.zeros(matting.shape, np.uint8)
+
+        cv2.drawContours(mask, contours, -1, 255, thickness)
+        return mask
+
+    def _create_contour_mask(self, gt_mask, pred_mask, line_width=10):
+        contour_mask = self._find_contours((gt_mask * 255).astype('uint8'), thickness=line_width) / 255.0
+        gt_contour_mask = gt_mask * contour_mask
+        pred_contour_mask = pred_mask * contour_mask
+        return gt_contour_mask, pred_contour_mask
+
+    def get_report(self):
+        return {self._metric: self._metric_sum / self._num_samples if self._num_samples else 0.0}
+
+
+class MeanIOUEvaluator(MattingEvaluatorBase):
+    """
+    Mean intersection-over-union evaluator
+    """
+
+    def __init__(self, metric='mIOU'):
+        super(MeanIOUEvaluator, self).__init__(metric=metric)
+
+    def add_predictions(self, predictions, targets):
+        """ Adding predictions and ground truth of images for image matting task
+        Args:
+            predictions: list of image matting predictions, [matting1, matting2, ...]. Shape: (N, ), type: PIL image object
+            targets: list of image matting ground truth, [gt1, gt2, ...]. Shape: (N, ), type: PIL image object
+        """
+
+        assert len(predictions) == len(targets)
+
+        num_class = 2
+        self._num_samples += len(predictions)
+        for pred_mask, gt_mask in zip(predictions, targets):
+            pred_binmask, gt_binmask = self._preprocess(pred_mask, gt_mask)
+            label = num_class * gt_binmask.astype('int') + pred_binmask
+            count = np.bincount(label.flatten(), minlength=num_class**2)
+            confusion_matrix = count.reshape(num_class, num_class)
+            iou = np.diag(confusion_matrix) / (confusion_matrix.sum(axis=1) + confusion_matrix.sum(axis=0) - np.diag(confusion_matrix) + 1e-10)
+            valid = confusion_matrix.sum(axis=1) > 0
+            mean_iou_per_image = np.nanmean(iou[valid])
+            self._metric_sum += mean_iou_per_image
+
+
+class ForegroundIOUEvaluator(MattingEvaluatorBase):
+    """
+    Foreground intersection-over-union evaluator
+    """
+
+    def __init__(self, metric='fgIOU'):
+        super(ForegroundIOUEvaluator, self).__init__(metric=metric)
+
+    def add_predictions(self, predictions, targets):
+        """ Adding predictions and ground truth of images for image matting task
+        Args:
+            predictions: list of image matting predictions, [matting1, matting2, ...]. Shape: (N, ), type: PIL image object
+            targets: list of image matting ground truth, [gt1, gt2, ...]. Shape: (N, ), type: PIL image object
+        """
+
+        assert len(predictions) == len(targets)
+
+        num_class = 2
+        self._num_samples += len(predictions)
+        for pred_mask, gt_mask in zip(predictions, targets):
+            pred_binmask, gt_binmask = self._preprocess(pred_mask, gt_mask)
+            if np.all(gt_binmask == 0):
+                res = 1 if np.all(pred_binmask == 0) else 0
+                self.metric_sum += res
+                continue
+
+            label = num_class * gt_binmask.astype('int') + pred_binmask
+            count = np.bincount(label.flatten(), minlength=num_class**2)
+            confusion_matrix = count.reshape(num_class, num_class)
+            iou = np.diag(confusion_matrix) / (confusion_matrix.sum(axis=1) + confusion_matrix.sum(axis=0) - np.diag(confusion_matrix) + 1e-10)
+            self._metric_sum += iou[1]
+
+
+class BoundaryMeanIOUEvaluator(MeanIOUEvaluator):
+    """
+    Boundary mean intersection-over-union evaluator
+    """
+
+    def __init__(self):
+        super(BoundaryMeanIOUEvaluator, self).__init__(metric='b_mIOU')
+
+    def _preprocess(self, pred_mask, gt_mask):
+        pred_mask = np.asarray(pred_mask)
+        gt_mask = np.asarray(gt_mask)
+        pred_binmask = self._convert2binary(pred_mask)
+        gt_binmask = self._convert2binary(gt_mask)
+        gt_boundary_mask, pred_boundary_mask = self._create_contour_mask(gt_binmask, pred_binmask)
+
+        return pred_boundary_mask.astype(np.int64), gt_boundary_mask.astype(np.int64)
+
+
+class BoundaryForegroundIOUEvaluator(ForegroundIOUEvaluator):
+    """
+    Boundary foreground intersection-over-union evaluator
+    """
+
+    def __init__(self):
+        super(BoundaryForegroundIOUEvaluator, self).__init__(metric='b_fgIOU')
+
+    def _preprocess(self, pred_mask, gt_mask):
+        pred_mask = np.asarray(pred_mask)
+        gt_mask = np.asarray(gt_mask)
+        pred_binmask = self._convert2binary(pred_mask)
+        gt_binmask = self._convert2binary(gt_mask)
+        gt_boundary_mask, pred_boundary_mask = self._create_contour_mask(gt_binmask, pred_binmask)
+
+        return pred_boundary_mask.astype(np.int64), gt_boundary_mask.astype(np.int64)
+
+
+class L1ErrorEvaluator(MattingEvaluatorBase):
+    """
+    L1 error evaluator
+    """
+
+    def __init__(self):
+        super(L1ErrorEvaluator, self).__init__(metric='L1Err')
+
+    def add_predictions(self, predictions, targets):
+        """ Adding predictions and ground truth of images for image matting task
+        Args:
+            predictions: list of image matting predictions, [matting1, matting2, ...]. Shape: (N, ), type: PIL image object
+            targets: list of image matting ground truth, [gt1, gt2, ...]. Shape: (N, ), type: PIL image object
+        """
+
+        assert len(predictions) == len(targets)
+
+        self._num_samples += len(predictions)
+        for pred_mask, gt_mask in zip(predictions, targets):
+            pred_mask = np.asarray(pred_mask)
+            gt_mask = np.asarray(gt_mask)
+            mean_l1 = np.abs(pred_mask.astype(np.float)-gt_mask.astype(np.float)).mean()
+            self._metric_sum += mean_l1
+
+
+class GroupWiseEvaluator(Evaluator):
+    """
+    Calculates metrics for the provided evaluator for each group separately.
+    """
+
+    def __init__(self, evaluator_ctor):
+        super().__init__()
+        self._evaluator_ctor = evaluator_ctor
+        self._evaluators = {}
+
+    def _get_id(self):
+        return 'group_wise_metrics'
+
+    def add_predictions(self, predictions, ground_truths):
+        """ Evaluate a batch of predictions.
+        Args:
+            predictions: predictions that can be consumed by target evaluator
+            ground_truths:
+            {
+                'targets': targets, # targets that can be consumed by the target evaluator
+                'groups': groups # group id can be either integer or string
+            }.
+        """
+
+        groups = ground_truths['groups']
+        targets = ground_truths['targets']
+        assert len(predictions) == len(groups) == len(targets)
+
+        preds_by_group = {}
+        gts_by_group = {}
+        for group, target, prediction in zip(groups, targets, predictions):
+            preds_by_group.setdefault(group, []).append(prediction)
+            gts_by_group.setdefault(group, []).append(target)
+
+        for group in preds_by_group:
+            group_preds = preds_by_group[group]
+            group_gts = gts_by_group[group]
+            if isinstance(predictions, np.ndarray):
+                group_preds = np.asarray(group_preds)
+            if isinstance(ground_truths['targets'], np.ndarray):
+                group_gts = np.asarray(group_gts)
+
+            self._evaluators.setdefault(group, self._evaluator_ctor()).add_predictions(group_preds, group_gts)
+
+    def get_report(self, **kwargs):
+        return {self._get_id(): {group: eval.get_report(**kwargs) for group, eval in self._evaluators.items()}}
